#include "SDCardManager.h"

SDCardManager::SDCardManager(const Pin csPin, uint32_t cfg) : _csPin(csPin), _spiCfg(cfg), _mounted(false)
{
  
}

float SDCardManager::getSize(const SizeUnit sizeUnit)
{
  uint64_t numberOf512BytesChunks = blocksPerCluster() * totalClusters();//cardSize();
  float result = 0;

  if(!isMounted()) return 0.0;
  
  switch(sizeUnit)
  {
    case KBIT:
      result = (float)numberOf512BytesChunks/2.0*8;
      break;
    case KBYTE:
      result = (float)numberOf512BytesChunks/2.0;
      break;
    case MBIT:
      result = (float)numberOf512BytesChunks/2.0/1024.0*8;
      break;
    case MBYTE:
      result = (float)numberOf512BytesChunks/2.0/1024.0;
      break;
    case GBIT:
      result = (float)numberOf512BytesChunks/2.0/1024.0/1024.0*8;
      break;
    case GBYTE:
      result = (float)numberOf512BytesChunks/2.0/1024.0/1024.0;
      break;
    default:
      result = (float)numberOf512BytesChunks/2.0/1024.0/1024.0;
  }
  return result;
}

boolean SDCardManager::mountSD()
{
  _mounted = this->begin(_csPin, _spiCfg);
  return _mounted;
}

void SDCardManager::unMountSD()
{
  _mounted = false;
  this->end();
}

boolean SDCardManager::isMounted()
{
  return _mounted;
}

CFGDictionary<CFGParameterValue> *SDCardManager::getCFGFile(const char *cfgFile)
{
  CFGFileParser cfgFileParser(*this, cfgFile);
  return (CFGDictionary<CFGParameterValue> *) cfgFileParser.parseFile();
}
